/*
 * main.c - main driver
 * (c) 2000 Zeljko Vrba (mordor@fly.srk.fer.hr)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

extern FILE *yyin;
extern int yyparse();
extern int yylineno;

#ifdef __GNUC__
static char rcsid[] __attribute__((unused)) = "$Id: main.c,v 1.1 2000-01-08 09:24:43+01 zeljko Exp $";
#else
static char rcsid[] = "$Id: main.c,v 1.1 2000-01-08 09:24:43+01 zeljko Exp $";
#endif

int opt_t = 0, opt_o = 0, opt_g = 0, opt_v = 0;
char *entrylib_path = NULL, *hpclib_path = NULL;

static void log(const char *fmt, ...)
{
	if(opt_v) {
		va_list ap;

		va_start(ap, fmt);
		vfprintf(stderr, fmt, ap);
		va_end(ap);
	}
}

static void usage(const char *prgnam)
{
	fprintf(stderr, "usage: %s [OPTIONS] source\n", prgnam);
	fprintf(stderr, " -e entries    entry point list\n");
	fprintf(stderr, " -i includes   include file\n");
	fprintf(stderr, " -o            optimize (this is VERY experimental)\n");
	fprintf(stderr, " -t            translate only\n");
	fprintf(stderr, " -v            be verbose\n");
	exit(1);
}

int main(int argc, char **argv)
{
	char optchar;
	char *libpath = getenv("HPCLIB");
	
	if(argc < 2) usage(argv[0]);
	
	entrylib_path = malloc(1024); hpclib_path = malloc(1024);
	if(!entrylib_path || !hpclib_path) {
		fprintf(stderr, "virtual memory exhausted");
		exit(1);
	}
	entrylib_path[0] = 0; hpclib_path[0] = 0;

	while((optchar = getopt(argc, argv, "e:i:otv")) != EOF) {
		switch(optchar) {
		case 'e': entrylib_path = optarg; break;
		case 'i': hpclib_path = optarg; break;
		case 't': opt_t = 1; break;
		case 'o': opt_o = 1; break;
		case 'v': opt_v = 1; break;
		default: usage(argv[0]);
		}
	}
	if(argv[optind] == NULL) usage(argv[0]);
	
	if(entrylib_path[0] == 0) {
		if(libpath != NULL) {
			sprintf(entrylib_path, "%s/entries.lib", libpath);
		} else {
			sprintf(entrylib_path, "%s/entries.lib", HPCLIB);
		}
	}
	log("entries file: %s\n", entrylib_path);
	
	if(hpclib_path[0] == 0) {
		if(libpath != NULL) {
			sprintf(hpclib_path, "%s/hpc.lib", libpath);
		} else {
			sprintf(hpclib_path, "%s/hpc.lib", HPCLIB);
		}
	}
	log("include file: %s\n", hpclib_path);

	log("compiling %s: ", hpclib_path);
	if((yyin = fopen(hpclib_path, "r")) != NULL) {
		yyparse();
		fclose(yyin);
		log("done\n");
	} else {
		log("can't open\n");
	}
	
	log("compiling %s: ", argv[optind]);
	if(!(yyin = fopen(argv[optind], "r"))) {
		fprintf(stderr, "can't open\n");
		exit(1);
	}

	/*	reset the line counter */
	yylineno = 1;
	yyparse();
	log("done\n");
	return 0;
}

