/*
 *	Syntax tree nodes and some macros (this is common include).
 */
#ifndef	__NODE_H__
#define	__NODE_H__

#define	FATAL(x)		\
	{ 					\
		fprintf(stderr,	\
				"internal compiler error %s: %s(%s:%d)\n",	\
				x, __FUNCTION__, __FILE__, __LINE__); \
		abort();		\
	}
				
/*	Possible variable (identifier) scope. */
enum scope { GLOBAL, LOCAL, PARAM };

/* 	The language type system. */
struct type {
	int		type_tag;					/* simple type tag */
	int 	scope, pos;					/* scope and declaration order */
	union {
		struct {
			struct type *elemtype;		/* the type of elements in array  */
			int ndims, dims[2];			/* number of dimensions and their */
		} array;						/* size */
		struct {
			int nargs;					/* number of arguments */
			struct type **args;			/* argument type */
			struct type *retarg;		/* return type, if FUNCTION */
			int external;				/* >=0 if declared as "external" */
		} subroutine;
	} class;
};

/*	Some additional opcodes for syntax tree, not defined in the grammar. */
enum
{
	PUSH = 2048, ANDTHEN, ORELSE, ARRGET, ARRPUT, RE2COMPLEX, DISPATCH,
	LIT_COMPLEX, LIT_LONGCOMPLEX
};

/*	This is the syntax tree node. */
struct st_node {
	int		opcode;				/* operation */
	int		line;				/* line in the source file */
	struct type	*type;			/* for type checks in expressions */
	union {
		union {					/* literals */
			double real;		/* common for REAL and LONGREAL */
			int	natural;
			int boolean;
			char *str;
			char *binary;
			char chr;
			double complex[2];	/* common for COMPLEX and LONGCOMPLEX */
		} literal;
		struct {				/* identifiers */
			char *name;			/* name */
		} identifier;
		struct {				/* statements and control structures */
								/* list of child nodes terminated by NULL */
			struct st_node *children[5];
		} control;
	} class;
};

/* 	Symbol table operations.  */
struct type* find_name(const char*);
int add_global_name(const char*, struct type*);
int add_local_name(const char*, struct type*);
void clear_local_names(void);
extern int globalsz, localsz;
extern char **global_names, **local_names;
extern struct type **global_data, **local_data;

/* 	Functions for allocation and initialization of nodes.  */
struct type* new_type(int, ...);
struct st_node* new_st_literal(int, ...);
struct st_node* new_st_identifier(const char*);
struct st_node* new_st_control(int, struct type*, ...);

#endif	// __NODE_H__

